#ifndef CGAMEENGINE
#define CGAMEENGINE

// ===================================================
// CGameEngine: Class. Defining the Game Engine, i.e. the behaviour of the game.
// ===================================================
class CGameEngine {
  private:
    int16_t  m_iPoints;
    uint32_t  m_iRoundStartTime;
    boolean   m_bRoundRunning;
    uint8_t   m_Phase;
  public:
    const uint8_t PHASE_RED = 0;
    const uint8_t PHASE_GREEN = 1;
    const uint8_t PHASE_BLUE = 2;

// ===================================================
// CGameEngine(): Constructor
// ===================================================
    CGameEngine() {
      m_iPoints = 0;
      m_iRoundStartTime = 0;
      m_bRoundRunning = false;
      m_Phase = PHASE_RED;
    }


// ===================================================
// CGameEngine::CGameEngine(int n_iPoints):
// Constructor. Sets the point count.
// ===================================================
    CGameEngine(int n_iPoints) {
      m_iPoints = n_iPoints;
      m_iRoundStartTime = 0;
      m_bRoundRunning = false;
      m_Phase = PHASE_RED;
    }


// ===================================================
// void CGameEngine::increasePoints(int iPoints):
// Increases the point count by the attrubute value.
// ===================================================
    void increasePoints(int iPoints) {
      m_iPoints += iPoints;
#ifdef DEBUG
      Serial.println(m_iPoints, DEC);
#endif
    }


// ===================================================
// void CGameEngine::decreasePoints(int iPoints):
// Decreases the point count by the attribute value.
// ===================================================
    void decreasePoints(int iPoints) {
      m_iPoints -= iPoints;
      if(m_iPoints < 0)
          m_iPoints = 0;
#ifdef DEBUG
      Serial.println(m_iPoints, DEC);
#endif
    }


// ===================================================
// uint16_t CGameEngine::getPoints:
// Returns the point count.
// ===================================================
    uint16_t getPoints() {
      return m_iPoints;
    }


// ===================================================
// void CGameEngine::setPoints(uint16_t n_iPoints):
// Gives the possibility to manipulate the point count.
// ===================================================
    void setPoints(uint16_t n_iPoints) {
      m_iPoints = n_iPoints;
    }


// ===================================================
// void CGameEngine::startRound():
// Starts a new round. Sets the required members.
// ===================================================
    void startRound() { 
        m_iPoints = 0;     
        m_iRoundStartTime = millis();
        m_bRoundRunning = true;
        m_Phase = PHASE_BLUE;
    }


// ===================================================
// boolean CGameEngine::checkRoundFinished():
// Checks whether the round is finished (time), and sets the members if necessary.
// ===================================================
    boolean checkRoundFinished() {
      boolean RoundFinished = false;
      if(m_bRoundRunning) {
        uint32_t iTimeElapsed = millis() - m_iRoundStartTime;
        if(iTimeElapsed > 60000) {
          m_bRoundRunning = false;
          RoundFinished = true;
        }
        
      }
      return RoundFinished;
    }


// ===================================================
// boolean CGameEngine::isRoundRunning()
// Returns m_bRoundRunning.
// ===================================================
    boolean isRoundRunning() {
      return m_bRoundRunning;
    }
  
};
#endif

