#ifndef CLANDINGCLEARANCE
#define CLANDINGCLEARANCE


// ===================================================
// CLandingClearance: Class. Defines whether we have a landing clearance or not.
// ===================================================
class CLandingClearance {
  
  private:
  boolean m_bClearedToLand;
  boolean m_bControlled;
  uint32_t m_iDepartureTime;
  uint32_t m_iTimeInAir;
  
  public:
  CLandingClearance() {
    m_bClearedToLand = false;
    m_bControlled = true;
    m_iDepartureTime = millis();
    m_iTimeInAir = 0;
  }

  ~CLandingClearance() {
    
  }

  boolean isCleared() {
    return m_bClearedToLand;
  }

  boolean isControlled() {
    uint32_t iTimeInAir = millis() - m_iDepartureTime;
    if(iTimeInAir < 5000) {       // If time in air < 5s, i.e. give the pilot 5 s to depart his quadrocopter
      m_bControlled = false;
      m_bClearedToLand = false;
    }
    else
    {
      m_bControlled = true;
      if(iTimeInAir != m_iTimeInAir)
      {
        if(!(iTimeInAir % 8000))      // If iTimeInAir has changed and can be divided by 8000: Toggle clearance state
        {
          m_bClearedToLand = !m_bClearedToLand;
          
        }
      }
    }

    m_iTimeInAir = iTimeInAir;
    
    return m_bControlled;
  }

  void ClearedToLand() {
    m_bClearedToLand = true;
  }

  void CancelLandingClearance() {
    m_bClearedToLand = false;
  }

  void setControlled(boolean n_bControlled) {
    m_bControlled = n_bControlled;
    if(!m_bControlled) {
      m_bClearedToLand = false;
    }
  }

  void afterTakeoff() {
    m_iDepartureTime = millis();
  }
    
  
};

#endif

