#include <SoftwareSerial.h>
#include <avr/pgmspace.h>
#include "CLandingClearance.h"
#include "CGameEngine.h"
#include "rgb.h"
#include "espUtil.h"

#define DEBUG

#define VERIFY_A  5809603.f
#define VERIFY_B       67.f
#define VERIFY_C    47060
#define GAME_ID      4736

#define LED_WLAN         13
                             // Pin 6 = AIN0 (+)
                             // Pin 7 = AIN1 (-)
#define PIN_ANGLE_SENSOR  2  // Pin 2 = Neigungssensor
#define PIN_POS_LIGHT     3  // Pin 3 = Pos. Light
#define PIN_START_SWITCH 10  // Pin 10= Start Switch

const char site[] PROGMEM = {"<html><head><title>Quadrocopter Landing Site</title></head><body style=\"background-color: black; color: green; font-weight: bold\"><h2 style=\"color: red; box-shadow: 3px 3px 3px 2px #444 inset; text-align: center\">Quadrocopter Challenge</h2>Aktuelle Highscores:<br><iframe src =\"http://www.gmhighscores.com/highscores_iframe.php?gameid=4736\" width=\"400\" height=\"300\"><p>Your browser does not support iframes.</p></iframe><br><p>Geben Sie den Spielernamen ein:<form method=\"get\"><input type=\"text\" name=\"playerName\" value=\"#playername#\" style=\"font-size: 14pt; color: blue\"><input type=\"submit\" value=\"Submit\" style=\"font-size: 14pt\"></form></p></body></html>"};
// #playername#


CLandingClearance landingClearance;
CGameEngine       gameEngine;
String            playerName;

// ===================================================
// void setup() : Startup actions. Press and hold the START button to bring the
//                board into a config Mode, where the barrier for the Analog Comparator
//                for the landing sensor can be set.
// ===================================================
void setup() {
  playerName = "unnamed";
    
  setupAnalogComparator();

  Serial.begin(19200);
  esp8266.begin(19200);

  esp8266.println("AT+RST");

  delay(5000);      // Give the ESP8266 enough time to initialize.

  pinMode(PIN_START_SWITCH, INPUT_PULLUP);
  pinMode(LED_WLAN, OUTPUT);
  pinMode(PIN_ANGLE_SENSOR, INPUT);
  pinMode(PIN_POS_LIGHT, OUTPUT);
  DDRC &= ~((1<<PINC0) | (1<<PINC1) | (1<<PINC2) | (1<<PINC3) | (1<<PINC4) | (1<<PINC5));   // All Pins are input pins (we want to use the internal pullup resistors)

  // Try all colors of the RBG LED to give the user a visual indication of the initialization
  switchOnBlue();
  delay(200);
  switchOffBlue();
  delay(200);
  switchOnRed();
  delay(200);
  switchOffRed();
  delay(200);
  switchOnGreen();
  delay(200);
  switchOffGreen();
  delay(200);
  
  digitalWrite(LED_WLAN, LOW); 
  analogWrite(PIN_POS_LIGHT, 0xff);

  if(!digitalRead(PIN_START_SWITCH))            // If the switch is pressed, enter the configuration mode
  {
    switchOffBlue();
    switchOnRed();
    switchOnGreen();
    // If start switch is pressed
    while(!digitalRead(PIN_START_SWITCH))       // Run while switch is depressed, so that the Poti can be adjusted, according to actual light conditions
    {
      if(isLanded())
      {
        switchOnBlue();
      }
      else
      {
        switchOffBlue();
      }
    }
    switchOffBlue();
    switchOffRed();
    switchOffGreen();
  }

#ifdef DEBUG
  Serial.println("Initializing...");
#endif

  boolean bInitSuccessful = true;

  bInitSuccessful &= espConfig();
  if(bInitSuccessful)
  {
    do { 
      
#ifdef DEBUG
    Serial.println("Trying to join AP:" + SSID);
#endif
    
    bInitSuccessful &= configStation();
    
#ifdef DEBUG
    if(!bInitSuccessful) Serial.println("Could not join AP!");
#endif

    } while(!bInitSuccessful);
    bInitSuccessful = true;
  }
  if(bInitSuccessful)
  {
#ifdef DEBUG    
    Serial.print("Initialized! WLAN joined! IP:");
    Serial.println( sendCmd("AT+CIPSTA?") );    
    Serial.print("Trying to ping 8.8.8.8...");
#endif
    bInitSuccessful &= sendCmd("AT+PING=\"8.8.8.8\"", "OK");        // Try to ping google...just to be sure, that there is a connection to the internet
    bInitSuccessful &= configTcpServer();
#ifdef DEBUG
    if(bInitSuccessful)
      Serial.println("successful!");
    else
      Serial.println("failed!");
#endif
  }
  else
  {
    Serial.println("Initialization failure!");
  }

  if(bInitSuccessful)
  {
    digitalWrite(LED_WLAN, HIGH);    
  }
  else {
    serialDebug();
  }


}


// ===================================================
// void setupAnalogComparator(): Sets the Analog Comparator up.
// ===================================================
void setupAnalogComparator()
{
  ACSR &= ~(1<<ACD);        // Clear ACD bit to switch Analog Comparator on
  ACSR &= ~(1<<ACBG);       // Switch off bandgap reference
  ACSR &= ~(1<<ACIE);       // Clear ACIE bit to switch off Interrupt
  ADCSRB &= ~(1<<ACME);     // Clear ACME bit to connect AIN1 to the negative input of the Analog Comparator
  DIDR1 |= ((1<<AIN1D) | (1<<AIN0D));
                            // Disable the Digital Input Buffers
}


boolean LastLanded = 0;
boolean LastAngleSensor = false;
boolean LastStartSwitch = true;


// ===================================================
// void loop(): This function is being run continuously, after the initialization was successfully completed.
// ===================================================
void loop() {
    
  uint8_t BrightnessPosLights = ((millis() >> 2) % 256);     // Range 0...255, calculate a dim value, depending on the time
  
  // If the game is actually running, let the "Position Lights" be illuminated and dimmed up and down continuously!
  if(gameEngine.isRoundRunning()) {

    if(BrightnessPosLights > 127)                           // If the value is > 127, start to dim the lights down
    {
      BrightnessPosLights = 255 - BrightnessPosLights;
    }

    BrightnessPosLights <<= 1;
  
    analogWrite(PIN_POS_LIGHT, BrightnessPosLights);    // Dim the lights up and down
  }
  else {
    analogWrite(PIN_POS_LIGHT, 0);                      // If the round is not running, switch the landing strip lights off
    
    if(LastStartSwitch != digitalRead(PIN_START_SWITCH))  // If the switch state has been changed
    {
      if(!digitalRead(PIN_START_SWITCH))                // switch is pressed
      {
        gameEngine.startRound();
      }
    }
  }
  LastStartSwitch = digitalRead(PIN_START_SWITCH);
  

  if( digitalRead(PIN_ANGLE_SENSOR) != LastAngleSensor )  // Trigger on the edge
  {
    if(!digitalRead(PIN_ANGLE_SENSOR))                    // Low is active, since there is a BC547 used as an inverter!
    {
    // if the stargate has been touched
      gameEngine.decreasePoints(100);
    }
  }
  LastAngleSensor = digitalRead(PIN_ANGLE_SENSOR);        // Store the last Value, so that we can trigger on the edge

  if(gameEngine.isRoundRunning()) {
    if(landingClearance.isControlled())
    {
      if(landingClearance.isCleared()) {
        switchOffBlue();
        switchOffRed();
        switchOnGreen();
      }
      else {
        switchOffBlue();
        switchOnRed();
        switchOffGreen();
      }
    }
    else {
       switchOnBlue();
       switchOffRed();
       switchOffGreen();
    }
  }

  if(gameEngine.isRoundRunning()) {
    if(LastLanded != isLanded())
    {
      if(isLanded())
      {
        if(landingClearance.isControlled())
        {
          if(landingClearance.isCleared())
          {
            gameEngine.increasePoints(100);
          }
          else
          {
            gameEngine.decreasePoints(100);
          }
        }
      }
      else
      {
          // After takeoff!!!
          landingClearance.afterTakeoff();        
      }
    }
  }
  LastLanded = isLanded();

  
  
  if( gameEngine.checkRoundFinished() )
  {
    
    // Round is finished, submit points
    uint32_t iVerifyAnswer = (uint32_t) ((((float)gameEngine.getPoints() + VERIFY_A) / VERIFY_B) + 0.5);
    while(iVerifyAnswer >= VERIFY_C)
      iVerifyAnswer -= VERIFY_C;

     if( commitHighscore(iVerifyAnswer) )
        configTcpServer();
#ifdef DEBUG   
  Serial.println("Points:");
  Serial.println(gameEngine.getPoints(), DEC);
#endif
  }


  // ===================================================
  // Webserver part
  // ===================================================
  if(esp8266.available())
  {
    // Data is available
    if(esp8266.find("+IPD,"))
    {
      int iConnectId = esp8266.parseInt();
      if(esp8266.findUntil("?playerName=", "\n"))
      {
                
        playerName = esp8266.readStringUntil(' ');
        playerName.replace("+"," ");
#ifdef DEBUG        
        Serial.print("?playerName : ");
        Serial.println(playerName);
#endif
      }
      
      if(sendWebsite(iConnectId) )
      {
#ifdef DEBUG
        Serial.println("Website sent! OK!");
#endif
      }
    }
    
  }
}


// ===================================================
// boolean isLanded(): Checks the value of the Analog Comparator in order to check, whether the Quadrocopter has landed.
//
// ===================================================
// true:  Quadrocopter landed
// false: Quadrocopter not landed
// ===================================================
boolean isLanded() {
  boolean ReturnValue = false;
  if(ACSR & (1<<ACO)) {
    // AIN0 > AIN1
#ifdef DEBUG
    //Serial.println("AIN0 > AIN1");
#endif
    ReturnValue = false;
  }
  else {
#ifdef DEBUG
    //Serial.println("AIN0 < AIN1");
#endif    
    ReturnValue = true;
  }
  return ReturnValue;
}


// ===================================================
// boolean sendWebsite(int iConnectionId): Prepare the website and send it to the client.
// ===================================================
// int iConnectionId: Connection Identifier, required since in the Server Mode we have CIPMUX=1, so multiple
//                    connections are possible, which have to be identified.
// ===================================================
// true: Website sent successfully.
// false:An error occured.
// ===================================================
boolean sendWebsite(int iConnectionId)
{
  boolean success = true;
  String webpage;  

  char tempChar;
  for(int i = 0; i < sizeof(site); i++)
  {
    tempChar = pgm_read_byte_near(site + i);
    webpage += tempChar;
  }
  

  webpage.replace("#playername#", playerName);

#ifdef DEBUG
  Serial.println("Site:");
  Serial.print(webpage);
#endif 

  if(sendCmd("AT+CIPSEND=" + String(iConnectionId) + "," + String(webpage.length()+2), ">"))
  {
    esp8266.println(webpage);
    esp8266.find("SEND OK");
    
    success &= true;
    success &= sendCmd("AT+CIPCLOSE=" + String(iConnectionId), "OK");
  }
  else
  {
    Serial.println("Error CIPSEND");
    success = false;
    sendCmd("AT+CIPCLOSE=" + String(iConnectionId));
  }
  return success;

  
}

// ===================================================
// boolean commitHighscore(uint32_t n_iVerifyAnswer): Commits the score of the actual round to the gamemaker Highsore service.
// ===================================================
// uint32_t n_iVerifyAnswer: Required for the GET-Query.
// ===================================================
// true: Highscore successfully sent.
// false:An error occured.
// ===================================================
boolean commitHighscore(uint32_t n_iVerifyAnswer)
{
  boolean success = true;

  success &= sendCmd("AT+CIPSERVER=0","OK");
  
  success &= configTcpClient();

  success &= sendCmd("AT+CIPSTART=\"TCP\",\"www.gmhighscores.com\",80","OK");
  
  String sHttpQuery = "GET /newhighscore_action.php?score=";
  sHttpQuery += String(gameEngine.getPoints());
  sHttpQuery += "&user=";
  sHttpQuery += playerName;
  sHttpQuery += "&verify=";
  sHttpQuery += String(n_iVerifyAnswer);
  sHttpQuery += "&game=";
  sHttpQuery += String(GAME_ID);
  sHttpQuery += " HTTP/1.1\r\nHost:www.gmhighscores.com\r\nUser-Agent:netreaddll\r\n\r\n";  
  
  if(success) {
    success &= sendCmd("AT+CIPSEND=" + String(sHttpQuery.length()), ">");    
  }

  if(success) {
    success &= sendCmd(sHttpQuery, "SEND OK");
  }

  if(success) {
    success &= sendCmd("AT+CIPCLOSE", "OK");
  }

  return success;
}


